Index: /trunk/SophyaPI/PIext/servnobjm.cc
===================================================================
--- /trunk/SophyaPI/PIext/servnobjm.cc	(revision 340)
+++ /trunk/SophyaPI/PIext/servnobjm.cc	(revision 341)
@@ -788,7 +788,4 @@
 if(expy.length()<=0) {nvar = 1; expy = "0.";}
 
-// $CHECK$ - cmv calculait le nombre d'entree ndata
-// en fonction de chaque objet - Je l'ai vire Reza 11/05/99
-
 // Creation NTuple Buffer
 char* ntn[4]= {"x","y","f","e"};
@@ -830,4 +827,5 @@
   double err_e, err_E;
   double stc2;
+  int nstep;
   int lp,lpg;
   int i1,i2,j1,j2;
@@ -867,4 +865,5 @@
 O.xc = O.yc = 0.;
 O.stc2 = 1.e-3;
+O.nstep = 100;
 O.err_e = O.err_E = -1.;
 O.lp = 1; O.lpg = 0;
@@ -919,4 +918,11 @@
   if(O.stc2<=0.) O.stc2 = 1.e-3;
 }
+if(strstr(opt.c_str(),",N")) { // Nombre maximum d'iterations
+  size_t p = opt.find(",N");
+  size_t q = opt.find_first_of(',',p+1);
+  string dum = opt.substr(p,q-p);
+  if(dum.length()>2) sscanf(dum.c_str(),",N%d",&O.nstep);
+  if(O.nstep<2) O.nstep = 100;
+}
 if(strstr(opt.c_str(),",l")) { // niveau de print
   size_t p = opt.find(",l");
@@ -926,5 +932,5 @@
   if(dum.length()>2) sscanf(dum.c_str(),",l%f",&ab);
   if(ab<0) ab = 0.;
-  O.lp = (int) ab; O.lpg = (int) 10.*(ab-O.lp);
+  O.lp = (int) ab; O.lpg = int(10.*(ab-(float)O.lp+0.01));
 }
 if(strstr(opt.c_str(),",I")) { // intervalle de fit selon X
@@ -950,44 +956,49 @@
 //| nom  : nom de l'objet qui peut etre:
 //|        fit-1D:  Vector,Histo1D,HProf ou GeneraFitData(1D)
-//|        fit-2D:  Matrix,Histo2D,Imagexx ou GeneraFitData(2D)
-//| func : pnn = fit polynome degre nn avec classe Poly (lineaire) 1D ou 2D
-//|      : Pnn = fit polynome degre nn avec GeneralFit (non-lineaire) 1D ou 2D
-//|      : gnn = fit gaussienne (hauteur) + polynome de degre nn 1D
-//|      : g   = fit gaussienne (hauteur) 1D
-//|      : enn = fit exponentielle + polynome de degre nn 1D
-//|      : e   = fit exponentielle 1D
-//|      : Gnn = fit gaussienne (volume) + polynome de degre nn 1D
-//|      : G   = fit gaussienne (volume) 1D
-//|      :     = fit gaussienne+fond (volume) 2D
-//|      : Gi  = fit gaussienne+fond integree (volume) 2D
-//|      : d   = fit DL de gaussienne+fond (volume) 2D
-//|      : di  = fit DL de gaussienne+fond integree (volume) 2D
-//|      : D   = fit DL de gaussienne+fond avec coeff variable p6 (volume) 2D
-//|      : Di  = fit DL de gaussienne+fond integree avec coeff variable p6 (volume) 2D
-//|      : M   = fit Moffat+fond (expos=p6) (volume) 2D
-//|      : Mi  = fit Moffat+fond integree (expos=p6) (volume) 2D
-//| par  : p1,...,pn valeur d'initialisation des parametres (def=0)
-//| step : s1,...,sn valeur des steps de depart (def=1)
-//| min  : m1,...,mn valeur des minima (def=1)
-//| max  : M1,...,Mn valeur des maxima (def=-1) (max<=min : pas de limite)
+//|        fit-2D:  Matrix,Histo2D,Image<T> ou GeneraFitData(2D)
+//| func : pnn : fit polynome degre nn avec classe Poly (lineaire) 1D ou 2D
+//|      : Pnn : fit polynome degre nn avec GeneralFit (non-lineaire) 1D ou 2D
+//|      : gnn : fit gaussienne (hauteur) + polynome de degre nn 1D
+//|      : g   : fit gaussienne (hauteur) 1D
+//|      : enn : fit exponentielle + polynome de degre nn 1D
+//|      : e   : fit exponentielle 1D
+//|      : Gnn : fit gaussienne (volume) + polynome de degre nn 1D
+//|      : G   : fit gaussienne (volume) 1D
+//|      :     : fit gaussienne+fond (volume) 2D
+//|      : Gi  : fit gaussienne+fond integree (volume) 2D
+//|      : d   : fit DL de gaussienne+fond (volume) 2D
+//|      : di  : fit DL de gaussienne+fond integree (volume) 2D
+//|      : D   : fit DL de gaussienne+fond avec coeff variable p6 (volume) 2D
+//|      : Di  : fit DL de gaussienne+fond integree avec coeff variable p6 (volume) 2D
+//|      : M   : fit Moffat+fond (expos=p6) (volume) 2D
+//|      : Mi  : fit Moffat+fond integree (expos=p6) (volume) 2D
+//| par  : p1,...,pn : valeur d'initialisation des parametres (def=0)
+//| step : s1,...,sn : valeur des steps de depart (def=1)
+//| min  : m1,...,mn : valeur des minima (def=1)
+//| max  : M1,...,Mn : valeur des maxima (def=-1) (max<=min : pas de limite)
 //| opt  : options "Eaa.b,eaa.b,f,r,caa.b,Xaa.b"
 //|       f = generation d'un Objet identique contenant la fonction fittee
 //|       r = generation d'un Objet identique contenant les residus
 //|       Xaa.b = aa.b valeur du DXi2 d'arret (def=1.e-3)
 //|       la.b = niveau "a.b" de print: a=niveau de print Fit1/2D
 //|                                     b=niveau de debug GeneralFit
 //|       Ii1/i2 numeros des bins X de l'histos utilises pour le fit [i1,i2]
 //|2D     Jj1/j2 numeros des bins Y de l'histos utilises pour le fit [j1,j2]
 //|       - L'erreur est celle associee a l'objet si existe, 1 sinon sauf si
 //|       E = erreur prise comme la racine de la valeur a fitter
 //|       Eaa.b = erreur prise aa.b*sqrt(val)
 //|       e = erreur prise egale a 1 pour toutes les valeurs
 //|       eaa.b = erreur prise egale a aa.b
 //|       xaa.b = demande de centrage: on fit x-aa.b au lieu de x)
 //|       x = demande de centrage: on fit x-xc au lieu de x
 //|           avec xc=abscisse du milieu de l'histogramme
 //|           Actif pour: exp+poly 1D, poly 1D
 //|                       gauss+poly 1D (mais xc est le centre de la gaussienne)
 //|2D     yaa.b et y = idem "xaa.b et x" mais pour y
 {
 AnyDataObj* obj=mOmg->GetObj(nom);
@@ -1073,5 +1084,5 @@
       <<" Cent="<<O.polcx<<","<<O.polcy<<","<<O.xc<<"+x"<<","<<O.yc<<"+y"
       <<" TypE="<<O.err_e<<","<<O.err_E
-      <<" StpX2="<<O.stc2
+      <<" StpX2="<<O.stc2<<" Nstep="<<O.nstep
       <<" lp,lpg="<<O.lp<<","<<O.lpg<<endl;
 
@@ -1101,5 +1112,5 @@
   // Gestion des erreurs a utiliser
   if(O.err_e>0.) e=O.err_e;
-  else if(O.err_E>0.) {e=(y<-1.||y>1.)?O.err_E*sqrt(fabs(y)):O.err_E;}
+  else if(O.err_E>0.) {e=(f<-1.||f>1.)?O.err_E*sqrt(fabs(f)):O.err_E;}
 
   // Remplissage de generalfit
@@ -1236,4 +1247,5 @@
 myfit.SetData(&mydata);
 myfit.SetStopChi2(O.stc2);
+myfit.SetMaxStep(O.nstep);
 {for(int i=0;i<myfunc->NPar();i++) {
   char str[10];
@@ -1242,17 +1254,19 @@
 }}
 if(O.lp>1) myfit.PrintFit();
-double c2r = (double) myfit.Fit();
+double c2r = -1.;
+int rcfit = (double) myfit.Fit();
 if(O.lp>0) myfit.PrintFit();
-if(c2r>0.) {
+if(rcfit>0) {
   c2r = myfit.GetChi2Red();
-  cout<<"C2r_Reduit = "<<c2r<<"  nstep="<<myfit.GetNStep()<<endl;
+  cout<<"C2r_Reduit = "<<c2r<<" nstep="<<myfit.GetNStep()<<" rc="<<rcfit<<endl;
   Vector ParFit(myfunc->NPar());
   for(int i=0;i<myfunc->NPar();i++) ParFit(i)=myfit.GetParm(i);
 } else {
-  cout<<"echec Fit, rc = "<<c2r<<"  nstep="<<myfit.GetNStep()<<endl;
+  cout<<"echec Fit, rc = "<<rcfit<<"  nstep="<<myfit.GetNStep()<<endl;
+  myfit.PrintFitErr(rcfit);
 }
 
 // Mise a disposition des resultats
-if(c2r>=0. && myfunc && (O.okres>0||O.okfun>0)) {
+if(rcfit>=0 && myfunc && (O.okres>0||O.okfun>0)) {
   string nomres = nom + "res";
   string nomfun = nom + "fun";
@@ -950,44 +956,49 @@
 //| opt  : options "Eaa.b,eaa.b,f,r,caa.b,Xaa.b"
-//|       f = generation d'un Objet identique contenant la fonction fittee
-//|       r = generation d'un Objet identique contenant les residus
-//|       Xaa.b = aa.b valeur du DXi2 d'arret (def=1.e-3)
-//|       la.b = niveau "a.b" de print: a=niveau de print Fit1/2D
-//|                                     b=niveau de debug GeneralFit
-//|       Ii1/i2 numeros des bins X de l'histos utilises pour le fit [i1,i2]
-//|2D     Jj1/j2 numeros des bins Y de l'histos utilises pour le fit [j1,j2]
-//|       - L'erreur est celle associee a l'objet si existe, 1 sinon sauf si
-//|       E = erreur prise comme la racine de la valeur a fitter
-//|       Eaa.b = erreur prise aa.b*sqrt(val)
-//|       e = erreur prise egale a 1 pour toutes les valeurs
-//|       eaa.b = erreur prise egale a aa.b
-//|       xaa.b = demande de centrage: on fit x-aa.b au lieu de x)
-//|       x = demande de centrage: on fit x-xc au lieu de x
-//|           avec xc=abscisse du milieu de l'histogramme
-//|           Actif pour: exp+poly 1D, poly 1D
-//|                       gauss+poly 1D (mais xc est le centre de la gaussienne)
-//|2D     yaa.b et y = idem "xaa.b et x" mais pour y
+//|      f : generation d'un Objet identique contenant la fonction fittee
+//|      r : generation d'un Objet identique contenant les residus
+//|      Xaa.b : aa.b valeur du DXi2 d'arret (def=1.e-3)
+//|      Naa : aa nombre maximum d'iterations (def=100)
+//|      la.b : niveau "a.b" de print: a=niveau de print Fit1/2D
+//|                                    b=niveau de debug GeneralFit
+//|      Ii1/i2 numeros des bins X de l'histos utilises pour le fit [i1,i2]
+//|2D    Jj1/j2 numeros des bins Y de l'histos utilises pour le fit [j1,j2]
+//|      - L'erreur est celle associee a l'objet (si elle existe),
+//|        elle est mise a 1 sinon, sauf si E... ou e... est precise:
+//|      Eaa.b : si |val|>=1 erreur = aa.b*sqrt(|val|)
+//|              si |val|<1  erreur = aa.b
+//|              si aa.b <=0 alors aa.b=1.0
+//|              E seul est equivalent a E1.0
+//|      eaa.b : erreur = aa.b
+//|              si aa.b <=0 alors aa.b=1.0
+//|              e seul est equivalent a e1.0
+//|      xaa.b : demande de centrage: on fit x-aa.b au lieu de x)
+//|      x : demande de centrage: on fit x-xc au lieu de x
+//|          avec xc=abscisse du milieu de l'histogramme
+//|          Actif pour exp+poly 1D, poly 1D
+//|                pour gauss+poly 1D, xc est le centre de la gaussienne.
+//|2D    yaa.b et y : idem "xaa.b et x" mais pour y
 {
 AnyDataObj* obj=mOmg->GetObj(nom);
